import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.event.*;

/*
 * FinchDisplay class
 * 
 * <P>
 * Create a frame that contains components for displaying Finch sensor
 * values and for picking a color for the nose LEDs
 * 
 * <P>
 * Specifically, the components are:
 * <OL>
 * <LI> A menu
 * <LI> Light sensors
 * <LI> Accelerometer
 * <LI> Obstacle sensors
 * <LI> Temperature sensor
 * <LI> RGB Color selector
 * <LI> A tool bar
 * </OL>
 * 
 */

public class FinchDisplay extends JFrame {

	// References to the text labels that will display sensor values
	private JLabel lightLeft;
	private JLabel lightRight;
	private JLabel accelX;
	private JLabel accelY;
	private JLabel accelZ;
	private JLabel obstacleLeft;
	private JLabel obstacleRight;
	private JLabel temperature;
	private JMenuBar menuBar;
	private JMenu fileMenu;
	private JMenuItem jmiExit;
	private JMenuItem jmiColorChooser;
	
	//	 Slider for Red LED
	private final JSlider slider1 = new JSlider(0, 255, 0);
	
	// Slider for Green LED
	private final JSlider slider2 = new JSlider(0, 255, 0);
	
	// Slider for Blue LED
	private final JSlider slider3 = new JSlider(0, 255, 0);
	
	// RGB values
	private int LED[];
	
	// Set to true if the window is open
	private boolean openFlag;

	
	/**
	 * Constructor of the Finch display frame.  Creates all of the
	 * frame contents.
	 *  
	 */
	
	public FinchDisplay()
	{	
		// Initialize the RGB LED array
		this.LED = new int[3];
		
		// Indicate that the window is open (or will soon be)
		this.openFlag = true;
		
		// Set up a grid that will have 1 column (and an arbitrary number of rows)
		this.setLayout(new GridLayout(0, 1, 10, 10));
		
		////////////////////////////////////////////////////////////
		// TODO: Add a menu
		// 
		
		// Create the menu bar
		this.menuBar = new JMenuBar();
		this.setJMenuBar(this.menuBar);
		
		// Create the menu
		this.fileMenu = new JMenu("Menu");
		this.menuBar.add(this.fileMenu);
		
		// Menu item 1: "Set Nose Color"
		this.jmiColorChooser = null;  // TODO: Replace "null"
		// TODO: add this menu item to the menu
		
		// Menu item 2: "Exit"
		this.jmiExit = null;  // TODO: Replace null
		// TODO: add this menu item to the menu

		
		// Set up action listener for nose color
		this.jmiColorChooser.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				colorChooser();
			}
		});
		
		// Set up action listener for exit
		// TODO: add action listener for jmiExit

		
		/////////////////////////////////////////////////////////
		// Light panel
		JPanel p1 = new JPanel(new GridLayout(0, 2, 10, 10));
		// Border for this pair of values
		p1.setBorder(new TitledBorder("Light Sensors"));

		// Value to be displayed.  Note, we save a 'handle' to this object
		//   so we can update it later.
		lightLeft = new JLabel("none");
		p1.add(lightLeft);
		
		// Value to be displayed
		lightRight = new JLabel("none");
		p1.add(lightRight);
		
		// Add this panel to the frame
		this.add(p1);
		
		/////////////////////////////////////////////////
		// Accelerometer panel
		JPanel Paccel = new JPanel(new GridLayout(0, 3, 10, 10));
		
		// Border for this pair of values
		Paccel.setBorder(new TitledBorder("Accelerometers"));
		
		// Create 3 different values
		accelX = new JLabel("none");
		Paccel.add(accelX);
		
		accelY= new JLabel("none");
		Paccel.add(accelY);
		
		accelZ = new JLabel("none");
		Paccel.add(accelZ);
		
		//		 Add this panel to the frame
		this.add(Paccel);
		
		/////////////////////////////////////////////////
		// Obstacle panel
		JPanel Pobstacle = new JPanel(new GridLayout(0, 2, 10, 10));
		// Border for this pair of values
		Pobstacle.setBorder(new TitledBorder("Obstacle"));

		// Create space for the left and right values
		obstacleLeft = new JLabel("none");
		Pobstacle.add(obstacleLeft);
		
		obstacleRight = new JLabel("none");
		Pobstacle.add(obstacleRight);
		
		//		 Add this panel to the frame
		this.add(Pobstacle);
		
		/////////////////////////////////////////////////
		// Temperature panel
		JPanel Ptemperature = new JPanel(new GridLayout(0, 1, 10, 10));
		// Border for this pair of values
		Ptemperature.setBorder(new TitledBorder("Temperature"));
		
		// Create the space to display the temperature
		temperature = new JLabel("none");
		Ptemperature.add(temperature);
		
		//		 Add this panel to the frame
		this.add(Ptemperature);
		
		/////////////////////////
		JPanel Pslider = new JPanel(new GridLayout(0, 1, 10, 10));
		
		
		
		// Set up a listener: stateChanged event is called when 
		//  the slider is moved.
		slider1.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				LED[0] = slider1.getValue();
			}
		});
		Pslider.add(slider1);
		

		
		// Set up a listener: stateChanged event is called when 
		//  the slider is moved.
		slider2.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				LED[1] = slider2.getValue();
			}
		});
		Pslider.add(slider2);
		

		
		// Set up a listener: stateChanged event is called when 
		//  the slider is moved.
		slider3.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				LED[2] = slider3.getValue();
			}
		});
		
		Pslider.add(slider3);
		
		// Finish off Slider panel configuration
		
		Pslider.setBorder(new TitledBorder("RGB"));
		this.add(Pslider);
		
		///////////////////////////////////
		// TODO: Add tool bar
		
		// Create ColorChooser button
		JButton jbtColorChooser = null; // TODO: use the colorchooser.gif icon
		
		// Add a listener to this button
		// TODO
		
		// Create Exit button
		JButton jbtExit = null;  // TODO: use the icon-exit.gif icon
		
		// Add a listener to this button
		// TODO
		
		// Create the tool bar 
		JToolBar jToolBar = new JToolBar("");
		
		// Add the buttons to the tool bar
		jToolBar.add(jbtColorChooser);
		jToolBar.add(jbtExit);
		
		// Add the tool bar to the bottom of this frame
		this.add(jToolBar, BorderLayout.SOUTH);
		
		//////////////////////////
		// Detect when the window is closed
		this.addWindowListener(new WindowListener(){
			public void windowOpened(WindowEvent event) {
			}
			public void windowDeactivated(WindowEvent event) {
			}
			public void windowActivated(WindowEvent event) {
			}
			public void windowDeiconified(WindowEvent event) {
			}
			public void windowIconified(WindowEvent event) {
			}
			public void windowClosing(WindowEvent event) {
				closeFinch();
				System.out.println("closing...");
			}
			public void windowClosed(WindowEvent event) {
			};
		});
	}
	
	/**
	 *  Method to be called in response to window being closed.
	 *   
	 *  <P>
	 *  Postconditions:
	 * <OL> 
	 * <LI>  The openFlag is set to false to indicate that the driver
	 * program should shut down
	 * </OL>
	 */
	private void closeFinch()
	{
		this.openFlag = false;
	}
	
	public boolean getOpenFlag()
	{
		return(this.openFlag);
	}
	
	/**
	 * Change the light sensor values that are displayed
	 * 
	 * @param left	The value of the left light sensor
	 * @param right The value of the right light sensor
	 * 
	 */
	public void setLight(Integer left, Integer right)
	{
		lightLeft.setText(left.toString());
		lightRight.setText(right.toString());
	}
	
	/**
	 * Change the accelerometer sensor values that are displayed
	 * 
	 * @param x	Acceleration in the x direction (front/back)
	 * @param y Acceleration in the y direction (lateral)
	 * @param z Acceleration in the z direction (up/down)
	 * 
	 */
	public void setAccel(Double x, Double y, Double z)
	{
		accelX.setText(x.toString());
		accelY.setText(y.toString());
		accelZ.setText(z.toString());
	}
	
	/**
	 * Change the obstacle sensor values that are displayed
	 * 
	 * @param left	The value of the left obstacle sensor
	 * @param right The value of the right obstacle sensor
	 * 
	 */
	public void setObstacle(Boolean left, Boolean right)
	{
		obstacleLeft.setText(left.toString());
		obstacleRight.setText(right.toString());
		
	}
	
	/**
	 * Change the temperature sensor value that is displayed
	 * 
	 * @param temp	Temperature sensor value
	 * 
	 */
	public void setTemperature(Double temp)
	{
		temperature.setText(temp.toString());
	}
	
	/**
	 * 
	 * 
	 * @return The RGB array that contains the current desired values of the LED.
	 */
	public int[] getLED(){
		return(LED);
	}
	
	
	/**
	 * TODO
	 * 
	 * <P>
	 * 
	 *  Open up a color chooser dialog box with a default color equal to 
	 *  the current state of the "LED" property.  If a new color is selected,
	 *  set the LED property AND the RGB slider bars to be equal to this new
	 *  color
	 *  
	 *  <P>
	 *  
	 *  Postconditions:
	 *  <OL>
	 *  <LI> slider1, slider2, slider3 have been changed to match the selected color
	 *  <LI> this.LED has been alterted to also match the selected color
	 *  </OL>
	 *
	 */
	private void colorChooser(){
		// Extract the default color
		Color color = new Color(LED[0], LED[1], LED[2]);
		
		// Show the color chooser dialog box
		color = null;  // TODO: replace null with a call to the color chooser dialog box
		
		// No color chosen?
		if(color == null)
			return;
		
		// Color has been selected: store the value
		// TODO: store in property LED
		// TODO: store in slider1, slider2, slider3
		// NOTE: DO NOT ACCESS THE FINCH HERE, ONLY MANIPULATE FinchDisplay properties
	}
	
	/**
	 * TODO
	 * 
	 * <P>
	 * 
	 *  Open a ConfirmDialog box that asks whether the user really wants to
	 *  quit the program.  If "Yes" is selected, then the program is shut
	 *  down by calling closeFinch()
	 *  
	 *  Postconditions:
	 *  <OL>
	 *  <LI> this.closeFinch() will have been called if indicated by the user.
	 *  </OL>
	 *
	 *<P>
	 *
	 *  Notes:
	 *  <OL>
	 *  <LI> JOptionPane.showConfirmDialog will return a "0" if "Yes" is selected.
	 *  <LI> The dialog box should only present two buttons: "Yes" and "No"
	 *  </OL>
	 *  
	 */
	private void exitChooser() {
		// Open the dialog box
		int ret = 0; // TODO: replace 0 with a call to showConfirmDialog()
		
		// If the answer was "Yes", take action to close the program
		// TODO

	}
}
